from __future__ import absolute_import, unicode_literals
import os
from typing import List
from celery import Celery
from fastapi_pundra.common.scheduler.schedule import bind_beat_schedule
from app.config.scheduler import schedules
from dotenv import load_dotenv

load_dotenv()

def create_celery_app(project_name: str, task_modules: List[str] | str = [], broker_type: str = 'redis'):
  # Convert task_modules to list if it's a string
  if isinstance(task_modules, str):
    task_modules = [task_modules]

  app = Celery(project_name)
  app.conf.update(
    broker_url=os.getenv('CELERY_BROKER_URL'),
    result_backend=os.getenv('CELERY_RESULT_BACKEND'),
    timezone='UTC',
    enable_utc=True,
  )

  if broker_type == 'redis':
    app.conf.beat_scheduler = 'redbeat.RedBeatScheduler'
    app.conf.redbeat_redis_url = os.getenv('CELERY_BROKER_URL')

  # Combine default task modules with provided task modules
  default_task_modules = ['fastapi_pundra.common.mailer.task']
  all_task_modules = default_task_modules + task_modules
  app.autodiscover_tasks(all_task_modules)

  app.conf.beat_schedule = bind_beat_schedule(schedules=schedules) 

  return app


from fastapi import Request
from urllib.parse import urlparse
import os
  
def paginate(request: Request, query, serilizer, the_page: int = 1, the_per_page: int = 10, wrap='data', additional_data=None):
    """Paginate the query.
    
    Args:
        request (Request): The FastAPI request object.
        query: The database query to paginate.
        serilizer: The serializer class to convert database objects.
        the_page (int, optional): Default page number. Defaults to 1.
        the_per_page (int, optional): Default items per page. Defaults to 10.
        wrap (str, optional): Key name for the data in response. Defaults to 'data'.
        additional_data (dict, optional): Additional data to include in the response. Defaults to None.
    """
    
    page = int(request.query_params.get('page', the_page))
    per_page = int(request.query_params.get('per_page', the_per_page))
    
    total = query.count()
    last_page = (total + per_page - 1) // per_page
    offset = (page - 1) * per_page
    paginated_query = query.offset(offset).limit(per_page).all()

    data = [serilizer.from_orm(item) for item in paginated_query]

    base_url = str(request.base_url)
    
    full_path = str(request.url)
    parsed_url = urlparse(full_path)
    path_without_query = f"{parsed_url.scheme}://{parsed_url.netloc}{parsed_url.path}"
    
    first_page_url = f"{path_without_query}?page=1&per_page={per_page}"
    last_page_url = f"{path_without_query}?page={last_page}&per_page={per_page}"
    next_page_url = f"{path_without_query}?page={page + 1}&per_page={per_page}" if page < last_page else None
    prev_page_url = f"{path_without_query}?page={page - 1}&per_page={per_page}" if page > 1 else None

    output = {
        'total': total,
        'per_page': per_page,
        'current_page': page,
        'last_page': last_page,
        'first_page_url': first_page_url,
        'last_page_url': last_page_url,
        'next_page_url': next_page_url,
        'prev_page_url': prev_page_url,
        'path': base_url,
        'kamal': os.getcwd(),
        'from': offset + 1 if data else None,
        'to': offset + len(data) if data else None,
        wrap: data
    }

    if additional_data:
        if callable(additional_data):
            output['additional_data'] = additional_data(data)
        else:
            output['additional_data'] = additional_data

    return output


from fastapi import Request
from pydantic import BaseModel, ValidationError
from functools import wraps
from fastapi_pundra.rest.helpers import the_query
from fastapi_pundra.rest.exceptions import ValidationException, BadRequestException

def dto(schema: BaseModel):
    def decorator(func):
        @wraps(func)
        async def wrapper(request: Request, *args, **kwargs):
            try:
                request_data = await the_query(request)
                validated_data = schema(**request_data)
                request.state.validated_data = validated_data
                return await func(request, *args, **kwargs)
            except ValidationError as e:
                errors = {}
                for error in e.errors():
                    field = error['loc'][0]
                    message = field + " " + error['msg']
                    if field not in errors:
                        errors[field] = []
                    errors[field].append(message)
                raise ValidationException(errors=errors)
            except ValueError:
                raise BadRequestException(message="Invalid JSON")
            
        return wrapper
    return decorator

__all__ = ['dto', 'setup_exception_handlers']


import strawberry
from typing import List, Optional, TypeVar, Generic

T = TypeVar('T')

@strawberry.type(description="Generic list type with pagination")
class PaginatedList(Generic[T]):
    data: List[T]
    total: int
    current_page: int
    next_page: Optional[int]
    prev_page: Optional[int]
    per_page: int
    last_page: int
    from_item: Optional[int]
    to_item: Optional[int]
    additional_data: Optional[dict]